@@ -37,6 +37,7 @@
 #include "types.h"
 #include "errors.h"
 #include "blowfish.h"
+#include "dynload.h"
 
 
 
@@ -454,7 +455,7 @@ decrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf )
 
 
 static const char*
-selftest()
+selftest(void)
 {
     BLOWFISH_context c;
     byte plain[] = "BLOWFISH";
