@@ -18,8 +18,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#define DEFINES_CIPHER_HANDLE 1
-
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -79,7 +77,7 @@ dummy_decrypt_block( void *c, byte *outbuf, byte *inbuf ) { BUG(); }
  * Put the static entries into the table.
  */
 static void
-setup_cipher_table()
+setup_cipher_table(void)
 {
 
     int i;
@@ -148,7 +146,7 @@ setup_cipher_table()
  * Try to load all modules and return true if new modules are available
  */
 static int
-load_cipher_modules()
+load_cipher_modules(void)
 {
     static int done = 0;
     static int initialized = 0;
@@ -160,6 +158,7 @@ load_cipher_modules()
     int any = 0;
 
     if( !initialized ) {
+	cipher_modules_constructor();
 	setup_cipher_table(); /* load static modules on the first call */
 	initialized = 1;
 	return 1;
