@@ -401,11 +401,8 @@ slow_poll(FILE *dbgfp, int dbgall, size_t *nbytes )
     fd_set fds;
   #if defined( __hpux )
     size_t maxFD = 0;
-    int pageSize = 4096;	/* PHUX doesn't have getpagesize() */
-  #elif defined( _M_XENIX ) || defined( __aux )
-    int maxFD = 0, pageSize = 4096;/* Nor do others, but they get fd right */
   #else
-    int maxFD = 0, pageSize = getpagesize();
+    int maxFD = 0;
   #endif /* OS-specific brokenness */
     int bufPos, i, usefulness = 0;
 
@@ -642,6 +639,9 @@ start_gatherer( int pipefd )
 		    select(0, NULL, NULL, NULL, &tv);
 		    continue;
 		}
+		if( errno == EPIPE ) /* parent has exited, so give up */
+		   exit(0);
+
 		/* we can't do very much here because stderr is closed */
 		if( dbgfp )
 		    fprintf(dbgfp, "gatherer can't write to pipe: %s\n",
@@ -756,7 +756,7 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDUNIX ($Revision: 1.9 $)";
+const char * const gnupgext_version = "RNDUNIX ($Revision: 1.11 $)";
 
 
 static struct {
