@@ -452,34 +452,19 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
 {
     int rc = 0;
     size_t len = 0;
-    unsigned emplines = afx->empty;
     int lastline = 0;
     unsigned maxlen, n;
     byte *p;
 
     len = 2;	/* reserve 2 bytes for the length header */
-    size -= 3;	/* and 1 for empline handling and 2 for the term header */
-		/* or the appended CR,LF */
+    size -= 2;	/* and 2 for the terminating header */
     while( !rc && len < size ) {
-	if( emplines ) {
-	    while( emplines && len < size ) {
-		buf[len++] = '\r';
-		buf[len++] = '\n';
-		emplines--;
-	    }
-	    continue;
-	}
-
+	/* copy what we have in the line buffer */
 	if( afx->faked == 1 )
-	    afx->faked++;  /* skip the first (empty) line */
+	    afx->faked++; /* skip the first (empty) line */
 	else {
 	    while( len < size && afx->buffer_pos < afx->buffer_len )
 		buf[len++] = afx->buffer[afx->buffer_pos++];
-	    if( afx->buffer_pos >= afx->buffer_len
-		&& !afx->not_dash_escaped ) {
-		buf[len++] = '\r';
-		buf[len++] = '\n';
-	    }
 	    if( len >= size )
 		continue;
 	}
@@ -490,13 +475,19 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
 	afx->buffer_len = iobuf_read_line( a, &afx->buffer,
 					   &afx->buffer_size, &maxlen );
 	if( !afx->buffer_len ) {
-	    rc = -1; /* eof */
+	    rc = -1; /* eof (should not happen) */
 	    continue;
 	}
 	if( !maxlen )
 	    afx->truncated++;
-	if( !afx->not_dash_escaped )
+	if( !afx->not_dash_escaped ) {
 	    afx->buffer_len = trim_trailing_ws( afx->buffer, afx->buffer_len );
+	    /* the buffer is always allocated with enough space to append
+	     * a CR, LF, Nul */
+	    afx->buffer[afx->buffer_len++] = '\r';
+	    afx->buffer[afx->buffer_len++] = '\n';
+	    afx->buffer[afx->buffer_len] = 0;
+	}
 	p = afx->buffer;
 	n = afx->buffer_len;
 
@@ -563,7 +554,6 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
 	afx->radbuf[0] = 0;
     }
 
-    afx->empty = emplines;
     *retn = len;
     return rc;
 }
@@ -608,15 +598,16 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 	    /* some mailers leave quoted-printable encoded characters
 	     * so we try to workaround this */
 	    if( afx->buffer_pos+2 < afx->buffer_len ) {
-		int c1, c2, c3;
-		c1 = afx->buffer[afx->buffer_pos];
-		c2 = afx->buffer[afx->buffer_pos+1];
-		c3 = afx->buffer[afx->buffer_pos+2];
-		if( isxdigit(c1) && isxdigit(c2) && strchr( "=\n\r\t ", c3 )) {
+		int cc1, cc2, cc3;
+		cc1 = afx->buffer[afx->buffer_pos];
+		cc2 = afx->buffer[afx->buffer_pos+1];
+		cc3 = afx->buffer[afx->buffer_pos+2];
+		if( isxdigit(cc1) && isxdigit(cc2)
+				  && strchr( "=\n\r\t ", cc3 )) {
 		    /* well it seems to be the case - adjust */
-		    c = isdigit(c1)? (c1 - '0'): (toupper(c1)-'A'+10);
+		    c = isdigit(cc1)? (cc1 - '0'): (toupper(cc1)-'A'+10);
 		    c <<= 4;
-		    c |= isdigit(c2)? (c2 - '0'): (toupper(c2)-'A'+10);
+		    c |= isdigit(cc2)? (cc2 - '0'): (toupper(cc2)-'A'+10);
 		    afx->buffer_pos += 2;
 		    afx->qp_detected = 1;
 		    goto again;
@@ -876,23 +867,26 @@ armor_filter( void *opaque, int control,
 	    iobuf_writestr(a, "-----");
 	    iobuf_writestr(a, head_strings[afx->what] );
 	    iobuf_writestr(a, "-----\n");
-	    iobuf_writestr(a, "Version: GnuPG v"  VERSION " ("
-					    PRINTABLE_OS_NAME ")\n");
+	    if( !opt.no_version )
+		iobuf_writestr(a, "Version: GnuPG v"  VERSION " ("
+					      PRINTABLE_OS_NAME ")\n");
 
 	    if( opt.comment_string ) {
 		const char *s = opt.comment_string;
-		iobuf_writestr(a, "Comment: " );
-		for( ; *s; s++ ) {
-		    if( *s == '\n' )
-			iobuf_writestr(a, "\\n" );
-		    else if( *s == '\r' )
-			iobuf_writestr(a, "\\r" );
-		    else if( *s == '\v' )
-			iobuf_writestr(a, "\\v" );
-		    else
-			iobuf_put(a, *s );
+		if( *s ) {
+		    iobuf_writestr(a, "Comment: " );
+		    for( ; *s; s++ ) {
+			if( *s == '\n' )
+			    iobuf_writestr(a, "\\n" );
+			else if( *s == '\r' )
+			    iobuf_writestr(a, "\\r" );
+			else if( *s == '\v' )
+			    iobuf_writestr(a, "\\v" );
+			else
+			    iobuf_put(a, *s );
+		    }
+		    iobuf_put(a, '\n' );
 		}
-		iobuf_put(a, '\n' );
 	    }
 	    else
 		iobuf_writestr(a,
