@@ -60,7 +60,7 @@ enum cmd_and_opt_values { aNull = 0,
     oDryRun	  = 'n',
     oOutput	  = 'o',
     oQuiet	  = 'q',
-    oRemote	  = 'r',
+    oRecipient	  = 'r',
     aSign	  = 's',
     oTextmodeShort= 't',
     oUser	  = 'u',
@@ -117,6 +117,7 @@ enum cmd_and_opt_values { aNull = 0,
     oDebugAll,
     oStatusFD,
     oNoComment,
+    oNoVersion,
     oCompletesNeeded,
     oMarginalsNeeded,
     oMaxCertDepth,
@@ -155,6 +156,8 @@ enum cmd_and_opt_values { aNull = 0,
     oEscapeFrom,
     oLockOnce,
     oKeyServer,
+    oEncryptTo,
+    oNoEncryptTo,
 aTest };
 
 
@@ -215,9 +218,12 @@ static ARGPARSE_OPTS opts[] = {
     { 301, NULL, 0, N_("@\nOptions:\n ") },
 
     { oArmor, "armor",     0, N_("create ascii armored output")},
+    { oRecipient, "recipient", 2, N_("|NAME|encrypt for NAME")},
+    { oRecipient, "remote-user", 2, "@"},  /* old option name */
+    { oEncryptTo, "encrypt-to", 2, "@" },
+    { oNoEncryptTo, "no-encrypt-to", 0, "@" },
   #ifdef IS_G10
     { oUser, "local-user",2, N_("use this user-id to sign or decrypt")},
-    { oRemote, "remote-user", 2, N_("use this user-id for encryption")},
     { oCompress, NULL,	      1, N_("|N|set compress level N (0 disables)") },
     { oTextmodeShort, NULL,   0, "@"},
     { oTextmode, "textmode",  0, N_("use canonical text mode")},
@@ -306,6 +312,7 @@ static ARGPARSE_OPTS opts[] = {
     { oRunAsShmCP, "run-as-shm-coprocess", 4, "@" },
     { oSetFilename, "set-filename", 2, "@" },
     { oComment, "comment", 2, "@" },
+    { oNoVersion, "no-version", 0,   "@"},
     { oNotDashEscaped, "not-dash-escaped", 0, "@" },
     { oEscapeFrom, "escape-from-lines", 0, "@" },
     { oLockOnce, "lock-once", 0, "@" },
@@ -538,6 +545,7 @@ main( int argc, char **argv )
     disable_core_dumps();
   #endif
     init_signals();
+    create_dotlock(NULL); /* register locking cleanup */
     i18n_init();
     opt.compress = -1; /* defaults to standard compress level */
     /* fixme: set the next two to zero and decide where used */
@@ -552,7 +560,7 @@ main( int argc, char **argv )
     opt.max_cert_depth = 5;
     opt.homedir = getenv("GNUPGHOME");
     if( !opt.homedir || !*opt.homedir ) {
-      #ifdef __MINGW32__
+      #ifdef HAVE_DRIVE_LETTERS
 	opt.homedir = "c:/gnupg";
       #else
 	opt.homedir = "~/.gnupg";
@@ -715,6 +723,7 @@ main( int argc, char **argv )
 			   opt.verbose = 0; opt.list_sigs=0; break;
 	  case oQuickRandom: quick_random_gen(1); break;
 	  case oNoComment: opt.no_comment=1; break;
+	  case oNoVersion: opt.no_version=1; break;
 	  case oCompletesNeeded: opt.completes_needed = pargs.r.ret_int; break;
 	  case oMarginalsNeeded: opt.marginals_needed = pargs.r.ret_int; break;
 	  case oMaxCertDepth: opt.max_cert_depth = pargs.r.ret_int; break;
@@ -757,20 +766,19 @@ main( int argc, char **argv )
 	  case oS2KDigest: s2k_digest_string = m_strdup(pargs.r.ret_str); break;
 	  case oS2KCipher: s2k_cipher_string = m_strdup(pargs.r.ret_str); break;
 
+	  case oNoEncryptTo: opt.no_encrypt_to = 1; break;
+	  case oEncryptTo: /* store the recipient in the second list */
+	    sl = add_to_strlist( &remusr, pargs.r.ret_str );
+	    sl->flags = 1;
+	    break;
 	#ifdef IS_G10
-	  case oRemote: /* store the remote users */
-	    sl = m_alloc( sizeof *sl + strlen(pargs.r.ret_str));
-	    strcpy(sl->d, pargs.r.ret_str);
-	    sl->next = remusr;
-	    remusr = sl;
+	  case oRecipient: /* store the recipient */
+	    add_to_strlist( &remusr, pargs.r.ret_str );
 	    break;
 	  case oTextmodeShort: opt.textmode = 2; break;
 	  case oTextmode: opt.textmode=1;  break;
 	  case oUser: /* store the local users */
-	    sl = m_alloc( sizeof *sl + strlen(pargs.r.ret_str));
-	    strcpy(sl->d, pargs.r.ret_str);
-	    sl->next = locusr;
-	    locusr = sl;
+	    add_to_strlist( &locusr, pargs.r.ret_str );
 	    break;
 	  case oCompress: opt.compress = pargs.r.ret_int; break;
 	  case oPasswdFD: pwfd = pargs.r.ret_int; break;
@@ -807,8 +815,12 @@ main( int argc, char **argv )
 	g10_exit(2);
 
     if( greeting ) {
-	tty_printf("%s %s; %s\n", strusage(11), strusage(13), strusage(14) );
-	tty_printf("%s\n", strusage(15) );
+	fprintf(stderr, "%s %s; %s\n",
+			strusage(11), strusage(13), strusage(14) );
+	fprintf(stderr, "%s\n", strusage(15) );
+      #ifdef IS_DEVELOPMENT_VERSION
+	log_info("NOTE: this is a development version!\n");
+      #endif
     }
 
     secmem_set_flags( secmem_get_flags() & ~2 ); /* resume warnings */
@@ -858,12 +870,6 @@ main( int argc, char **argv )
 	log_error(_("invalid S2K mode; must be 0, 1 or 3\n"));
     }
 
-    {	const char *p = strusage(13);
-	for( ; *p && (isdigit(*p) || *p=='.'); p++ )
-	    ;
-	if( *p )
-	    log_info("NOTE: this is a development version!\n");
-    }
 
     if( log_get_errorcount(0) )
 	g10_exit(2);
