@@ -83,15 +83,13 @@ keygen_add_std_prefs( PKT_signature *sig, void *opaque )
 
     keygen_add_key_expire( sig, opaque );
 
-    buf[0] = CIPHER_ALGO_BLOWFISH;
+    buf[0] = CIPHER_ALGO_TWOFISH;
     buf[1] = CIPHER_ALGO_CAST5;
     build_sig_subpkt( sig, SIGSUBPKT_PREF_SYM, buf, 2 );
 
     buf[0] = DIGEST_ALGO_RMD160;
     buf[1] = DIGEST_ALGO_SHA1;
-    buf[2] = DIGEST_ALGO_TIGER;
-    buf[3] = DIGEST_ALGO_MD5;
-    build_sig_subpkt( sig, SIGSUBPKT_PREF_HASH, buf, 4 );
+    build_sig_subpkt( sig, SIGSUBPKT_PREF_HASH, buf, 2 );
 
     buf[0] = 2;
     buf[1] = 1;
@@ -490,7 +488,7 @@ ask_keysize( int algo )
 
 
 static u32
-ask_expire_interval()
+ask_expire_interval(void)
 {
     char *answer;
     int valid_days=0;
