@@ -72,7 +72,7 @@ int encrypt_filter( void *opaque, int control,
 /*-- sign.c --*/
 int complete_sig( PKT_signature *sig, PKT_secret_key *sk, MD_HANDLE md );
 int sign_file( STRLIST filenames, int detached, STRLIST locusr,
-	       int encrypt, STRLIST remusr, const char *outfile );
+	       int do_encrypt, STRLIST remusr, const char *outfile );
 int clearsign_file( const char *fname, STRLIST locusr, const char *outfile );
 
 /*-- sig-check.c --*/
