@@ -41,7 +41,7 @@ typedef struct {
 	 char *ret_str;
      } r;		    /* Return values */
      struct {
-	 int index;
+	 int idx;
 	 int inarg;
 	 int stopped;
 	 const char *last;
@@ -127,10 +127,12 @@ const char *strusage( int level );
 
 
 /*-- dotlock.c --*/
-const char *make_dotlock( const char *file_to_lock, long timeout );
-int	    release_dotlock( const char *lockfile );
-
+struct dotlock_handle;
+typedef struct dotlock_handle *DOTLOCK;
 
+DOTLOCK create_dotlock( const char *file_to_lock );
+int make_dotlock( DOTLOCK h, long timeout );
+int release_dotlock( DOTLOCK h );
 
 
 /*-- fileutil.c --*/
